/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
package com.patrikdufresne.jface.databinding.viewers;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Arrays;
import java.util.Date;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.observable.Observables;
import org.eclipse.core.databinding.observable.Realm;
import org.eclipse.jface.databinding.swt.DisplayRealm;
import org.eclipse.jface.databinding.viewers.ObservableListContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;

/**
 * A simple TableViewer to demonstrate usage
 * 
 * @author Karl St-Cyr
 */
public class ColumnSupportTableViewerExample {
    public class MyModel {

        protected PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);

        public int counter;

        public Date time;

        public MyModel(int counter, Date time) {
            this.counter = counter;
            this.time = time;
        }

        /**
         * Java bean function.
         * 
         * @param listener
         *                     The PropertyChangeListener to be added
         */
        public void addPropertyChangeListener(PropertyChangeListener listener) {
            this.changeSupport.addPropertyChangeListener(listener);
        }

        public int getCounter() {
            return this.counter;
        }

        public String getName() {
            return "Item " + this.counter;
        }

        public Date getTime() {
            return this.time;
        }

        /**
         * Java bean function.
         * 
         * @param listener
         *                     The PropertyChangeListener to be removed
         */
        public void removePropertyChangeListener(PropertyChangeListener listener) {
            this.changeSupport.removePropertyChangeListener(listener);
        }

        public void setCounter(int counter) {
            this.changeSupport.firePropertyChange("counter", this.counter, this.counter = counter);
        }

        public void setTime(Date time) {
            this.changeSupport.firePropertyChange("time", this.time, this.time = time);
        }

    }

    /**
     * @param args
     */
    public static void main(String[] args) {
        final Display display = new Display();
        Realm.runWithDefault(DisplayRealm.getRealm(display), new Runnable() {
            @Override
            public void run() {
                Shell shell = new Shell(display);
                shell.setLayout(new FillLayout());
                new ColumnSupportTableViewerExample(shell);
                shell.open();
                while (!shell.isDisposed()) {
                    if (!display.readAndDispatch()) display.sleep();
                }
            }
        });
        display.dispose();
    }

    public ColumnSupportTableViewerExample(final Composite comp) {
        DataBindingContext dbc = new DataBindingContext();
        final TableViewer v = new TableViewer(new Table(comp, SWT.BORDER));
        v.setContentProvider(new ObservableListContentProvider());
        v.getTable().setHeaderVisible(true);
        MyModel[] model = createModel();
        ColumnSupport.create(v, "Name", MyModel.class, "name").setPixelLayoutData(100).addTextEditingSupport(dbc);
        ColumnSupport.create(v, "Counter", MyModel.class, "counter").setWeightLayoutData(1, 10).addTextEditingSupport(dbc);
        ColumnSupport.create(v, "Time", MyModel.class, "time").setWeightLayoutData(1, 10).addTextEditingSupport(dbc);
        v.getTable().setLinesVisible(true);
        v.setInput(Observables.staticObservableList(Arrays.asList(model)));
    }

    private MyModel[] createModel() {
        MyModel[] elements = new MyModel[10];
        for (int i = 0; i < 10; i++) {
            elements[i] = new MyModel(i, new Date());
        }
        return elements;
    }
}