/**
 * Copyright (C) 2019 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package com.patrikdufresne.managers.databinding;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.UpdateListStrategy;
import org.eclipse.core.databinding.UpdateSetStrategy;
import org.eclipse.core.databinding.UpdateValueStrategy;
import org.eclipse.core.databinding.observable.list.IObservableList;
import org.eclipse.core.databinding.observable.set.IObservableSet;
import org.eclipse.core.databinding.observable.value.IObservableValue;

import com.patrikdufresne.managers.Managers;

/**
 * Specialised data binding context for managers.
 * 
 * @author Patrik Dufresne
 *
 */
public class ManagerDataBindingContext extends DataBindingContext {

    private Managers managers;

    /**
     * Create a data binding context for the given managers.
     * 
     * @param managers
     *            the managers used for persistance.
     */
    public ManagerDataBindingContext(Managers managers) {
        this.managers = managers;
    }

    /**
     * Returns an update value strategy to be used for copying values from the from value to the to value. Clients may
     * override.
     *
     * @param fromValue
     * @param toValue
     * @return a update value strategy
     */
    @Override
    protected <T, M> UpdateValueStrategy<T, M> createTargetToModelUpdateValueStrategy(IObservableValue<T> fromValue, IObservableValue<M> toValue) {
        return new ManagerUpdateValueStrategy(managers);
    }

    /**
     * @param targetObservableList
     * @param modelObservableList
     * @return an update list strategy
     */
    @Override
    protected <T, M> UpdateListStrategy<T, M> createTargetToModelUpdateListStrategy(
            IObservableList<T> targetObservableList,
            IObservableList<M> modelObservableList) {
        return new ManagerUpdateListStrategy(managers);
    }

    /**
     * @param targetObservableSet
     * @param modelObservableSet
     * @return a default set update strategy
     */
    @Override
    protected <T, M> UpdateSetStrategy<T, M> createTargetToModelUpdateSetStrategy(IObservableSet<T> targetObservableSet, IObservableSet<M> modelObservableSet) {
        return new ManagerUpdateSetStrategy(managers);
    }

}
